<?php

/**
 *
 * $Id: class.TaskApplicationMailSettings.php,v 1.2 2005/04/24 02:56:59 dgorski Exp $
 *
 * This file is part of coWiki. coWiki is free software under the terms of
 * the GNU General Public License (GPL). Read the LICENSE file. If you did
 * not receive a copy of the license and are not able to obtain it through
 * the internet, please send a note to <license@cowiki.org> so we can mail
 * you a copy immediately.
 *
 * @package     htdocs
 * @subpackage  setup
 * @access      public
 *
 * @author      Daniel T. Gorski, <daniel.gorski@develnet.org>
 * @copyright   (C) Daniel T. Gorski, {@link http://www.develnet.org}
 * @license     http://www.gnu.org/licenses/gpl.html
 * @version     $Revision: 1.2 $
 *
 */

/**
 * coWiki - Installer task class
 *
 * @package     htdocs
 * @access      public
 *
 * @author      Daniel T. Gorski, <daniel.gorski@develnet.org>
 * @since       coWiki 0.4.0
 */
class TaskApplicationMailSettings extends AbstractTask {

    const STRING_TEST_MAIL = 'Send a test mail';

    // --------------------------------------------------------------------

    public function __construct($Tpl) {
        parent::__construct($Tpl);
        parent::checkValidUser();
        parent::loadClassLoader();

        // ---

        // Load configuration data only once. Keep it in sesssion (state
        // holder) then
        if (!$this->State->hasMailData()) {

            // Load application configuration
            $Conf = $this->getConfig();

            // Initialize state holder values
            $this->State->setMailFullName(
                $Conf->getSectionKeyValue('MAIL', 'FULL_NAME')
            );
            $this->State->setMailReturnPath(
                $Conf->getSectionKeyValue('MAIL', 'RETURN_PATH')
            );
            $this->State->setMailAbusePath(
                $Conf->getSectionKeyValue('MAIL', 'ABUSE_PATH')
            );

            $this->State->setHasMailData(true);
        }
    }

    // --------------------------------------------------------------------

    public function isRequired() {
        return true;
    }

    // --------------------------------------------------------------------

    public function validate() {

        // Is mail() function disabled? If the Mailer class can send
        // directly via SMTP without a local mail transfer agent (MTA),
        // this check can/should be skipped.
        if (!function_exists('mail')) {
            $this->addWarning(
                'Your installation does not support sending emails via
                PHPs mail() function. This function has probably been
                disabled by your hoster. No emails can be send!'
            );
            return true;
        }

        // ----------------------------------------------------------------

        // Store submitted values
        $this->State->setMailFullName(@$_POST['fullname']);
        $this->State->setMailReturnPath(@$_POST['return']);
        $this->State->setMailAbusePath(@$_POST['abuse']);

        // Complain about stupid settings (e.g. example settings taken
        // directly for the core.conf-dist file)
        if (isset($_POST['fullname']) && $_POST['fullname'] == '') {
            $this->addErr('Please enter your full name. This is important.');
        }
        if (isset($_POST['return'])) {
            if ($_POST['return'] == ''
                || strrpos($_POST['return'], '@') === false
                || substr($_POST['return'], -12) == '@example.com') {
                $this->addErr('The return path address is missing or wrong.');
            }
        }
        if (isset($_POST['abuse'])) {
            if ($_POST['abuse'] == ''
                || strrpos($_POST['abuse'], '@') === false
                || substr($_POST['abuse'], -12) == '@example.com') {
                $this->addErr('The abuse report address is missing or wrong.');
            }
        }

        if ($this->hasMessage()) {
            return false;
        }

        // ----------------------------------------------------------------

        // Test email functionality? Off we go ..
        if (@$_POST['submit'] == self::STRING_TEST_MAIL) {

            $sStr =   'Hello, this looks very good.' . "\n\n";

            $sStr .=  'This is a manually triggered ' . COWIKI_FULL_IDENT;
            $sStr .=  ' test email, that has been delivered by the';
            $sStr .=  ' '.COWIKI_NAME . ' installer';
            $sStr .=  ' to your email address. It is a technical';
            $sStr .=  ' email capability and flawlessness check of your';
            $sStr .=  ' ' . COWIKI_NAME;
            $sStr .=  ' installation.' . "\n";
            $sStr .=  "\n";

            $sStr .=  'Your settings are:';
            $sStr .=  "\n\n";

            $sStr .=  ' * Your name: '.$this->State->getMailFullName();
            $sStr .=  "\n";
            $sStr .=  ' * Return-Path: '.$this->State->getMailReturnPath();
            $sStr .=  "\n";
            $sStr .=  ' * Abuse reports would go to: ';
            $sStr .=  $this->State->getMailAbusePath();
            $sStr .=  "\n\n";

            $sStr .=  'If these settings are correct, you may delete this';
            $sStr .=  ' email as the delivery basically worked and';
            $sStr .=  ' you can proceed further with the installer.';
            $sStr .=  "\n\n\n";

            // The following is a joke and can (should?) be changed. It is
            // not ment seriously.
            $sStr .=  'Respectfully,';
            $sStr .=  ' your mechanically devoted installer automaton.';
            $sStr .=  "\n\n";
            $sStr .=  str_repeat('-', 72);
            $sStr .=  "\n\n";

            $sStr .=  'P.S.';
            $sStr .=  ' If you didn\'t expect this email, or think you are';
            $sStr .=  ' the wrong recipient, you will get';
            $sStr .=  ' more information about the coWiki software';
            $sStr .=  ' and the responsible persons at';
            $sStr .=  ' <'.COWIKI_HOME.'>.';

            $sStr .=  "\n\n";
            
            $sStr .=  'Important: Please consider that the sender of';
            $sStr .=  ' this email is none of the coWiki development team,';
            $sStr .=  ' but a person who tries to install this software';
            $sStr .=  ' on his/her machine. The email header and the sender';
            $sStr .=  ' information attached below, will help you to track';
            $sStr .=  ' the sender. Please do not complain of it to the';
            $sStr .=  ' coWiki developer team. Complain to the sender.';
            $sStr .=  ' Thank you.';

            // ---

            $Mail = new Mailer();
            $Mail->setIsSenderInfoEnabled(true);
            $Mail->setReturnPath($this->State->getMailReturnPath());

            $Mail->setFrom(
                'coWiki setup',
                $this->State->getMailReturnPath()
            );
            $Mail->addTo(
                $this->State->getMailFullName(),
                $this->State->getMailReturnPath()
            );

            $Mail->addXHeader('X-Abuse', $this->State->getMailAbusePath());

            $Mail->setSubject(
                '['.COWIKI_NAME.']'
                .' Setup test email (' . COWIKI_FULL_IDENT . ')'
            );
            $Mail->setBody($Mail->formatString($sStr));
            $Mail->send();

            // ---

            return $this->addInfo(
                'A test email has been send to the address '
                . '&lt;' . $this->State->getMailReturnPath() . '&gt;. If
                you\'ve received it, you may continue with the setup.'
            );
        }

        return true;
    }

    // --------------------------------------------------------------------

    public function render() {

        $sStr = 'coWiki sends emails, e.g. if a user wants to be notified
                 about changes in his document or as posting reply. To
                 warrant civilized and polite correspondence with your
                 users and keep peace with your postmaster, you will have
                 to set these entries:

                 <ul>
                  <li>
                    Your full name: This is the string that will be
                    attached to automatic emails that your system sends.
                    E.g.
                    <i>Respectfully yours, &lt;Your full name here&gt;</i>
                  </li>
                  <li>
                    The so called \'Return-Path\': If an email can not be
                    delivered, it will come back to this account. If you
                    do not set this return path, you\'ll get troubles with
                    your postmaster or and/or your ISP. This email address
                    can be a normal address you usually use.
                  </li>
                  <li>
                    The abuse path: If someone wants to complain about
                    anything regarding coWikis emails, you want to talk to
                    him first, before he talks to your ISP, right? This
                    email address can be a normal address you use and can
                    be the same as the Return-Path above.
                  </li>
                 </ul>

                 You can test the technical flawlessness by hitting the
                 \'<b>'.self::STRING_TEST_MAIL.'</b>\' button: you should
                 get a confirmation email delivered to the account you
                 specified in the \'Return-Path\' field.';

        $sStr .=  '<br/><br/>';

        $sStr .=  '<table align="center" cellspacing="0"
                    cellpadding="0" border="0">';
        $sStr .=    '<tr>';

        $sStr .=      '<td>';
        $sStr .=        '<fieldset>';
        $sStr .=          '<legend>coWiki email settings</legend>';

        $sStr .=          '<table cellpadding="2" cellspacing="0"';
        $sStr .=          ' border="0" class="fieldset-content">';

        $sStr .=            '<tr>';
        $sStr .=              '<td nowrap align="right">';
        $sStr .=                'Your full name:';
        $sStr .=              '</td>';
        $sStr .=              '<td colspan="3">';
        $sStr .=                '<input type="text" name="fullname"';
        $sStr .=                ' size="35" value="';
        $sStr .=                  $this->State->getMailFullName();
        $sStr .=                '">';
        $sStr .=              '</td>';
        $sStr .=            '</tr>';

        $sStr .=            '<tr>';
        $sStr .=              '<td nowrap align="right">';
        $sStr .=                'Return-/Bounce path:';
        $sStr .=              '</td>';
        $sStr .=              '<td colspan="3">';
        $sStr .=                '<input type="text" name="return"';
        $sStr .=                ' size="35" value="';
        $sStr .=                  $this->State->getMailReturnPath();
        $sStr .=                '">';
        $sStr .=              '</td>';
        $sStr .=            '</tr>';

        $sStr .=            '<tr>';
        $sStr .=              '<td nowrap align="right">';
        $sStr .=                'Abuse report address:';
        $sStr .=              '</td>';
        $sStr .=              '<td colspan="3">';
        $sStr .=                '<input type="text" name="abuse"';
        $sStr .=                ' size="35" value="';
        $sStr .=                  $this->State->getMailAbusePath();
        $sStr .=                '">';
        $sStr .=              '</td>';
        $sStr .=            '</tr>';

        $sStr .=            '<tr>';
        $sStr .=              '<td colspan="4" align="right"';
        $sStr .=                ' style="padding-top:8px">';
        $sStr .=                '<input type="submit" name="submit"';
        $sStr .=                ' class="submit" value="';
        $sStr .=                  self::STRING_TEST_MAIL;
        $sStr .=                '">';
        $sStr .=              '</td>';
        $sStr .=            '</tr>';

        $sStr .=          '</table>';
        $sStr .=        '</fieldset>';
        $sStr .=      '</td>';

        $sStr .=    '</tr>';
        $sStr .=  '</table>';

        $sStr .=  '<br/>';

        // Define content
        $this->Tpl->assign('CONTENT', $sStr);

        // Define informational/warning/error messages
        $this->Tpl->assign('MESSAGE', $this->getMsgQueueFormatted());

        echo $this->Tpl->parse(SETUP_PATH.'/tpl/window.default.tpl');
    }

} // of class

?>