<?php

/**
 *
 * $Id: class.TaskDocumentStorageContainerChooser.php,v 1.5 2005/07/14 23:40:07 dgorski Exp $
 *
 * This file is part of coWiki. coWiki is free software under the terms of
 * the GNU General Public License (GPL). Read the LICENSE file. If you did
 * not receive a copy of the license and are not able to obtain it through
 * the internet, please send a note to <license@cowiki.org> so we can mail
 * you a copy immediately.
 *
 * @package     htdocs
 * @subpackage  setup
 * @access      public
 *
 * @author      Daniel T. Gorski, <daniel.gorski@develnet.org>
 * @copyright   (C) Daniel T. Gorski, {@link http://www.develnet.org}
 * @license     http://www.gnu.org/licenses/gpl.html
 * @version     $Revision: 1.5 $
 *
 */

/**
 * coWiki - Installer task class
 *
 * @package     htdocs
 * @access      public
 *
 * @author      Daniel T. Gorski, <daniel.gorski@develnet.org>
 * @since       coWiki 0.4.0
 */
class TaskDocumentStorageContainerChooser extends AbstractTask {

    private
        $Conf = null;

    // --------------------------------------------------------------------

    public function __construct($Tpl) {
        parent::__construct($Tpl);
        parent::checkValidUser();
        parent::loadClassLoader();

        // ---

        // Load configuration data only once. Keep it in sesssion (state
        // holder) then
        if (!$this->State->hasDocStorageScheme()) {

            // Load application configuration
            $Conf = $this->getConfig();

            // Initialize state holder values
            $UriInfo = new UriInfo(
                          $Conf->getSectionKeyValue(
                            '.DOCUMENT', 'RESOURCE'
                          )
                       );

            $this->State->setDocStorageScheme($UriInfo->get('scheme'));

            $this->State->setShallCreateNewDocumentDatabase(
                $this->State->isInitialSetup()
            );

            $this->State->setShallCreateNewConnectorUser(
                $this->State->isInitialSetup()
            );

            $this->State->setHasDocStorageScheme(true);
        }
    }

    // --------------------------------------------------------------------

    public function isRequired() {
        return true;
    }

    // --------------------------------------------------------------------

    public function validate() {

        // Store submitted values
        $this->State->setDocStorageScheme(@$_POST['scheme']);
        $this->State->setShallCreateNewDocumentDatabase(
            @$_POST['create'] == 'new'
        );
        $this->State->setShallCreateNewConnectorUser(
            @$_POST['create'] == 'new'
        );
        return true;
    }

    // --------------------------------------------------------------------

    public function render() {

        $sStr =   'Determine where to store the main coWiki document data.
                  Depending on if you are installing coWiki for the first
                  time, and on the storage container (database) that you
                  are going to use, an administrative root-user access to
                  the database is required to setup the appropriate
                  connector user with his credentials, privileges and to
                  initialize the basic table scheme.
                  <br/><br/>

                  If you lack of administrative access and are not allowed
                  to create a new database, ask your database administrator
                  to grant you the required privileges or let him create
                  the database and the appropriate connector user for you.
                  <br/><br/>

                  <b class="warn">It is strongly recommended <i>not</i> to
                  use MyISAM storage engine if you are using the MySQL
                  database. Choose the InnoDB storage engine instead!</b>';

        $sStr .=  '<br/><br/>';

        $sStr .=  '<table width="90%" align="center" cellspacing="0"
                    cellpadding="0" border="0">';
        $sStr .=    '<tr>';

        $sStr .=      '<td>';
        $sStr .=        '<fieldset>';
        $sStr .=          '<legend>coWiki document storage engine</legend>';

        $sStr .=          '<table cellpadding="2" cellspacing="0"';
        $sStr .=          ' border="0" class="fieldset-content">';

        $sStr .=            '<tr>';
        $sStr .=              '<td nowrap align="right">';
        $sStr .=                'Container:';
        $sStr .=              '</td>';
        $sStr .=              '<td colspan="2">';
        $sStr .=                '<select name="scheme" style="width:100%">';

        foreach ($this->getConnectorSchemes() as $k => $v) {
            $sStr .=              '<option value="'.$k.'"';
            if ($this->State->getDocStorageScheme() == $k) {
                $sStr .=  ' selected="true"';
            }
            $sStr .=              '>';
            $sStr .=                $v;
            $sStr .=              '</option>';
        }

        $sStr .=                '</select>';
        $sStr .=              '</td>';
        $sStr .=            '</tr>';

        // Horizontal separator
        $sStr .=            '<tr>';
        $sStr .=              '<td colspan="3">';
        $sStr .=                SETUP_FIELDSET_SEPARATOR;
        $sStr .=              '</td>';
        $sStr .=            '</tr>';

        $sStr .=            '<tr valign="top">';
        $sStr .=              '<td></td>';
        $sStr .=              '<td nowrap align="right">';
        $sStr .=                '<input id="existing" type="radio"';
        $sStr .=                ' name="create" value="existing"';

        if (!$this->State->shallCreateNewDocumentDatabase()) {
            $sStr .=  ' checked="true"';
        }

        $sStr .=                '>';

        $sStr .=              '</td>';
        $sStr .=              '<td>';
        $sStr .=                '<label for="existing">';
        $sStr .=                  'Use an already existing coWiki document
                                  database with an existing and set up
                                  connector user.';
        $sStr .=                '</label>';
        $sStr .=              '</td>';
        $sStr .=            '</tr>';

        $sStr .=            '<tr valign="top">';
        $sStr .=              '<td></td>';
        $sStr .=              '<td nowrap align="right">';
        $sStr .=                '<input id="new" type="radio"';
        $sStr .=                ' name="create" value="new"';

        if ($this->State->shallCreateNewDocumentDatabase()) {
            $sStr .=  ' checked="true"';
        }

        $sStr .=                '>';
        $sStr .=              '</td>';
        $sStr .=              '<td>';
        $sStr .=                '<label for="new">';
        $sStr .=                  'Create a completely new document database.
                                  Administrative access to the database is
                                  required.';
        $sStr .=                '</label>';
        $sStr .=              '</td>';
        $sStr .=            '</tr>';

        $sStr .=          '</table>';
        $sStr .=        '</fieldset>';
        $sStr .=      '</td>';

        $sStr .=    '</tr>';
        $sStr .=  '</table>';

        $sStr .=  '<br/>';

        // Define content
        $this->Tpl->assign('CONTENT', $sStr);

        // Define informational/warning/error messages
        $this->Tpl->assign('MESSAGE', $this->getMsgQueueFormatted());

        echo $this->Tpl->parse(SETUP_PATH.'/tpl/window.default.tpl');
    }

} // of class

?>