<?php

/**
 *
 * $Id: class.Migrator_20050513_02.php,v 1.3 2005/07/14 23:50:10 dgorski Exp $
 *
 * This file is part of coWiki. coWiki is free software under the terms of
 * the GNU General Public License (GPL). Read the LICENSE file. If you did
 * not receive a copy of the license and are not able to obtain it through
 * the internet, please send a note to <license@cowiki.org> so we can mail
 * you a copy immediately.
 *
 * @package     htdocs
 * @subpackage  setup
 * @access      public
 *
 * @author      Daniel T. Gorski, <daniel.gorski@develnet.org>
 * @copyright   (C) Daniel T. Gorski, {@link http://www.develnet.org}
 * @license     http://www.gnu.org/licenses/gpl.html
 * @version     $Revision: 1.3 $
 *
 */

/**
 * Migrator implementation
 *
 * @package     htdocs
 * @access      public
 *
 * @author      Daniel T. Gorski, <daniel.gorski@develnet.org>
 * @since       coWiki 0.4.0
 */
class Migrator_20050513_02 implements MigratorInterface {

    private
        $State = null;

    // --------------------------------------------------------------------

    public function __construct($StateHolder) {
        $this->State = $StateHolder;
    }

    // --------------------------------------------------------------------

    /**
     *  Get a short description of what this Migrator is going to do
     */
    public function getDescription() {
        return 'Convert MyISAM tables to InnoDB.';
    }

    // --------------------------------------------------------------------

    /**
     *  Check if this Migrator has to be performed, its task is:
     *
     *  Convert legacy MyISAM tables to InnoDB.
     */
    public function isRequired() {

        // Get the resource DSN (database connection details)
        $UriInfo = new UriInfo($this->State->getDocumentStorageDSN());

        // Do we use MySQL? Check if this Migrator can/should generally
        // handle the requested connector scheme. If not, this Migrator is
        // "not required".
        switch ($UriInfo->get('scheme')) {

/* Disabled. See ChangeLog entry 2005-07-13
            case 'mysql':
            case 'mysqli':
*/
            case 'mysql+innodb':
            case 'mysqli+innodb':
                break;

            default:
                // Not a known MySQL scheme, Migrator not required
                return false;
        }

        // Check if we have a valid storage connection with a valid
        // connector user and a valid database schema. If we do not, the
        // "basic" Migrators, those that create the connector user and
        // install the database schema were not executed yet. In this case
        // we assume that they will be executed first and report that this
        // very Migrator is required, although it was not able to check
        // its status (yet).
        if (!$this->State->hasValidDocStorageConnection()) {
            return true;
        }

        // ----------------------------------------------------------------

        // Check if re-definition to InnoDB is required
        $Storage = StorageFactory::getInstance()->create($UriInfo);        
        $Storage->useDatabase($UriInfo->get('basepath'));

        $rResult = $Storage->query('SHOW TABLE STATUS LIKE "cowiki_%"');

        while ($aData = $Storage->fetchArray($rResult)) {
            if (isset($aData['Engine']) && $aData['Engine'] == 'MyISAM') {
                return true;
            }
        }

        return false;
    }

    // --------------------------------------------------------------------

    /**
     *  Perform the Migrator task
     */
    public function perform() {

        // Get the connector user resource DSN (database connection details)
        $UriInfo = new UriInfo($this->State->getDocumentStorageDSN());

        // Connect to the storage/select database
        $Storage = StorageFactory::getInstance()->create($UriInfo);        
        $Storage->useDatabase($UriInfo->get('basepath'));

        // Apply changes
        $aArr = array (
            'ALTER TABLE cowiki_comment TYPE=INNODB',
            'ALTER TABLE cowiki_group TYPE=INNODB',
            'ALTER TABLE cowiki_node TYPE=INNODB',
            'ALTER TABLE cowiki_node_hist TYPE=INNODB',
            'ALTER TABLE cowiki_node_hist_spill TYPE=INNODB',
            'ALTER TABLE cowiki_node_ref TYPE=INNODB',
            'ALTER TABLE cowiki_node_spill TYPE=INNODB',
            'ALTER TABLE cowiki_shoutbox TYPE=INNODB',
            'ALTER TABLE cowiki_user TYPE=INNODB',
            'ALTER TABLE cowiki_user_group TYPE=INNODB'
        );
        
        $Storage->begin();

        foreach ($aArr as $sQuery) {
            $Storage->query($sQuery);
        }

        $Storage->commit();
    }

} // of class

?>